import { RegExpOptions } from '@terminal-nerds/snippets-regexp/creator';
import { PascalCase, Join } from 'type-fest';
import { z } from 'zod';

declare const CHAR_TYPES: readonly ["latin", "number", "special"];
type CharType = (typeof CHAR_TYPES)[number];
declare const LOWER_CASED_LATIN_CHARS: readonly ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "r", "s", "t", "u", "v", "w", "x", "y", "z"];
type LowerCaseLatinChar = (typeof LOWER_CASED_LATIN_CHARS)[number];
declare const UPPER_CASED_LATIN_CHARS: readonly ("A" | "B" | "C" | "D" | "E" | "F" | "G" | "H" | "I" | "J" | "K" | "L" | "M" | "N" | "O" | "P" | "R" | "S" | "T" | "U" | "V" | "W" | "X" | "Y" | "Z")[];
type UpperCaseLatinChar = PascalCase<LowerCaseLatinChar>;
declare const LATIN_CHARS: readonly ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "r", "s", "t", "u", "v", "w", "x", "y", "z", ...("A" | "B" | "C" | "D" | "E" | "F" | "G" | "H" | "I" | "J" | "K" | "L" | "M" | "N" | "O" | "P" | "R" | "S" | "T" | "U" | "V" | "W" | "X" | "Y" | "Z")[]];
type LatinChar = (typeof LATIN_CHARS)[number];
declare const NUMBER_CHARS: readonly ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9"];
type NumberChar = (typeof NUMBER_CHARS)[number];
declare const SPECIAL_CHARS: readonly ["~", "`", "@", "#", "$", "%", "^", "&", "*", "(", ")", "-", "_", "+", "=", "\\", "|", "]", "}", "[", "{", "\"", "'", ";", ":", "/", "?", ".", ">", ",", "<"];
type SpecialChar = (typeof SPECIAL_CHARS)[number];
declare const SINGLE_CHARS: readonly ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "r", "s", "t", "u", "v", "w", "x", "y", "z", "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "~", "`", "@", "#", "$", "%", "^", "&", "*", "(", ")", "-", "_", "+", "=", "\\", "|", "]", "}", "[", "{", "\"", "'", ";", ":", "/", "?", ".", ">", ",", "<"];
type SingleChar<T extends string = string> = T | LatinChar | NumberChar | SpecialChar;
declare const CHARS: {
    readonly latin: readonly ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "r", "s", "t", "u", "v", "w", "x", "y", "z"];
    readonly number: readonly ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9"];
    readonly special: readonly ["~", "`", "@", "#", "$", "%", "^", "&", "*", "(", ")", "-", "_", "+", "=", "\\", "|", "]", "}", "[", "{", "\"", "'", ";", ":", "/", "?", ".", ">", ",", "<"];
};
type Char<T extends CharType> = T extends "latin" ? LatinChar : T extends "number" ? NumberChar : T extends "special" ? SpecialChar : never;
declare const NON_LATIN_CHARS: readonly ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "~", "`", "@", "#", "$", "%", "^", "&", "*", "(", ")", "-", "_", "+", "=", "\\", "|", "]", "}", "[", "{", "\"", "'", ";", ":", "/", "?", ".", ">", ",", "<"];
declare const NON_NUMBER_CHARS: readonly ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "r", "s", "t", "u", "v", "w", "x", "y", "z", ...("A" | "B" | "C" | "D" | "E" | "F" | "G" | "H" | "I" | "J" | "K" | "L" | "M" | "N" | "O" | "P" | "R" | "S" | "T" | "U" | "V" | "W" | "X" | "Y" | "Z")[], "~", "`", "@", "#", "$", "%", "^", "&", "*", "(", ")", "-", "_", "+", "=", "\\", "|", "]", "}", "[", "{", "\"", "'", ";", ":", "/", "?", ".", ">", ",", "<"];
declare const NON_SPECIAL_CHARS: readonly ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "r", "s", "t", "u", "v", "w", "x", "y", "z", ...("A" | "B" | "C" | "D" | "E" | "F" | "G" | "H" | "I" | "J" | "K" | "L" | "M" | "N" | "O" | "P" | "R" | "S" | "T" | "U" | "V" | "W" | "X" | "Y" | "Z")[], "0", "1", "2", "3", "4", "5", "6", "7", "8", "9"];
declare const CHAR_SCHEMA: z.ZodString;
declare const CHAR_TYPE_SCHEMA: z.ZodEnum<["latin", "number", "special"]>;
declare const LATIN_CHAR_SCHEMA: z.ZodEnum<["a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "r", "s", "t", "u", "v", "w", "x", "y", "z"]>;
declare const NUMBER_CHAR_SCHEMA: z.ZodEnum<["0", "1", "2", "3", "4", "5", "6", "7", "8", "9"]>;
declare const SPECIAL_CHAR_SCHEMA: z.ZodEnum<["~", "`", "@", "#", "$", "%", "^", "&", "*", "(", ")", "-", "_", "+", "=", "\\", "|", "]", "}", "[", "{", "\"", "'", ";", ":", "/", "?", ".", ">", ",", "<"]>;
declare const CHARS_SCHEMAS: {
    readonly latin: z.ZodEnum<["a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "r", "s", "t", "u", "v", "w", "x", "y", "z"]>;
    readonly number: z.ZodEnum<["0", "1", "2", "3", "4", "5", "6", "7", "8", "9"]>;
    readonly special: z.ZodEnum<["~", "`", "@", "#", "$", "%", "^", "&", "*", "(", ")", "-", "_", "+", "=", "\\", "|", "]", "}", "[", "{", "\"", "'", ";", ":", "/", "?", ".", ">", ",", "<"]>;
};
declare function validateSingleChar(char: string): asserts char is SingleChar;
declare function isSingleChar(input: string): input is SingleChar;
declare function getJoinedChars<T extends readonly string[]>(chars: T): Join<T, "">;
interface CharOptions<T extends CharType> extends RegExpOptions {
    type: T;
}
declare function validateCharType(type: string): asserts type is CharType;
declare function isValidCharType(type: string): type is CharType;
declare function getCharType<T extends SingleChar>(char: T, 
/** @defaultValue `{ caseInsensitive: true }` */
options?: Pick<CharOptions<CharType>, "caseInsensitive">): CharType;
declare function isChar<T extends CharType>(char: string, 
/** @defaultValue `{ caseInsensitive: true }` */
options: Pick<CharOptions<T>, "type" | "caseInsensitive">): char is Char<T>;
declare function hasChars<T extends CharType>(input: string, 
/** @defaultValue `{ caseInsensitive: true }` */
options: CharOptions<T>): boolean;
declare function getChars<T extends CharType>(input: string, 
/** @defaultValue `{ caseInsensitive: true, global: true, multiline: true }` */
options: CharOptions<T>): Array<Char<T>>;

export { CHARS, CHARS_SCHEMAS, CHAR_SCHEMA, CHAR_TYPES, CHAR_TYPE_SCHEMA, Char, CharOptions, CharType, LATIN_CHARS, LATIN_CHAR_SCHEMA, LOWER_CASED_LATIN_CHARS, LatinChar, LowerCaseLatinChar, NON_LATIN_CHARS, NON_NUMBER_CHARS, NON_SPECIAL_CHARS, NUMBER_CHARS, NUMBER_CHAR_SCHEMA, NumberChar, SINGLE_CHARS, SPECIAL_CHARS, SPECIAL_CHAR_SCHEMA, SingleChar, SpecialChar, UPPER_CASED_LATIN_CHARS, UpperCaseLatinChar, getCharType, getChars, getJoinedChars, hasChars, isChar, isSingleChar, isValidCharType, validateCharType, validateSingleChar };
